package com.ym.mec.biz.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.auth.api.enums.SysUserType;
import com.ym.mec.auth.api.enums.UserLockFlag;
import com.ym.mec.biz.dal.dao.StudentRegistrationDao;
import com.ym.mec.biz.dal.dto.StudentApplyDetailDto;
import com.ym.mec.biz.dal.dto.StudentFeeDto;
import com.ym.mec.biz.dal.entity.StudentRegistration;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.biz.dal.page.StudentRegistrationQueryInfo;
import com.ym.mec.biz.service.StudentRegistrationService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;

@Service
public class StudentRegistrationServiceImpl extends BaseServiceImpl<Long, StudentRegistration> implements StudentRegistrationService {

    @Resource
    private StudentRegistrationDao studentRegistrationDao;
    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Override
    public BaseDAO<Long, StudentRegistration> getDAO() {
        return studentRegistrationDao;
    }

    @Override
    public PageInfo<StudentApplyDetailDto> queryStudentDetailPage(StudentRegistrationQueryInfo queryInfo) {
        PageInfo<StudentApplyDetailDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentApplyDetailDto> dataList = null;
        int count = studentRegistrationDao.queryStudentDetailCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentRegistrationDao.queryStudentDetailPage(params);
        }
        if (count == 0) {
            dataList = new ArrayList<>();
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    public boolean batchUpdateSubject(String userIds, Integer subId) {
        String[] userId = userIds.split(",");
        StudentRegistration studentRegistration;
        Date date = new Date();
        for (String str : userId) {
            studentRegistration = studentRegistrationDao.get(Long.parseLong(str));
            if (studentRegistration.getPaymentStatus() == YesOrNoEnum.NO) {
                studentRegistration.setUpdateTime(date);
                studentRegistration.setSubjectId(subId);
                studentRegistrationDao.update(studentRegistration);
            }
        }
        return true;
    }

    @Override
    public StudentFeeDto queryFeeDetail(Integer studentId, String musicGroupId) {
//		return studentRegistrationDao.queryFeeDetail(studentId,musicGroupId);
        return null;
    }

    @Override
    public List<StudentRegistration> getNoClassStuBySubjectId(Integer musicGroupId, Integer actualSubjectId) {
        return studentRegistrationDao.getNoClassStuBySubjectId(musicGroupId, actualSubjectId);
    }

    @Override
    public Integer getNoClassStuCountBySubjectId(Integer musicGroupId, Integer actualSubjectId) {
        return studentRegistrationDao.getNoClassStuCountBySubjectId(musicGroupId, actualSubjectId);
    }

    @Override
    public Integer updateByUserIdAndMusicGroupId(StudentRegistration studentRegistration) {
        return studentRegistrationDao.updateByUserIdAndMusicGroupId(studentRegistration);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public StudentRegistration addStudent(StudentRegistration studentRegistration) {
        Date date = new Date();
        SysUser sysUser = sysUserFeignService.getUserByMobile(studentRegistration.getParentsPhone());
        if (sysUser == null) {
            sysUser = new SysUser();
            sysUser.setPhone(studentRegistration.getParentsPhone());
            sysUser.setCreateTime(date);
            sysUser.setUpdateTime(date);
            sysUser.setLockFlag(UserLockFlag.NOACTIVATED);
            sysUser.setDelFlag(com.ym.mec.auth.api.enums.YesOrNoEnum.NO);
            sysUser.setUserType(SysUserType.STUDENT);
            sysUser.setGender(studentRegistration.getGender());
            sysUser.setOrganId(studentRegistration.getOrganId());
            sysUser.setRealName(studentRegistration.getName());
            sysUser.setIdCardNo(studentRegistration.getIdCardNo());
            sysUserFeignService.addUser(sysUser);
        }
        studentRegistration.setUserId(sysUser.getId());
        studentRegistrationDao.insert(studentRegistration);
        return studentRegistration;
    }
}