package com.ym.mec.education.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.ym.mec.education.base.PageResponse;
import com.ym.mec.education.entity.ClassGroup;
import com.ym.mec.education.entity.ClassGroupStudentMapper;
import com.ym.mec.education.entity.MusicGroupStudentFee;
import com.ym.mec.education.entity.SysUser;
import com.ym.mec.education.mapper.ClassGroupStudentMapperMapper;
import com.ym.mec.education.req.ClassGroupReq;
import com.ym.mec.education.resp.ClassStudentResp;
import com.ym.mec.education.resp.StudentListResp;
import com.ym.mec.education.service.IClassGroupService;
import com.ym.mec.education.service.IClassGroupStudentMapperService;
import com.ym.mec.education.service.IMusicGroupStudentFeeService;
import com.ym.mec.education.service.ISysUserService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * <p>
 * 班级、学生关系表 服务实现类
 * </p>
 *
 * @author lemeng
 * @since 2019-09-25
 */
@Service("IClassGroupStudentMapperService")
public class ClassGroupStudentMapperServiceImpl extends ServiceImpl<ClassGroupStudentMapperMapper, ClassGroupStudentMapper> implements IClassGroupStudentMapperService {

    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private IClassGroupService classGroupService;
    @Autowired
    private IMusicGroupStudentFeeService musicGroupStudentFeeService;


    @Override
    public PageResponse getPage(ClassGroupReq classGroupReq) {
        if (Objects.isNull(classGroupReq.getGroupId())) {
            return PageResponse.errorParam();
        }
        ClassGroup classGroup = classGroupService.getById(classGroupReq.getGroupId());
        Page<ClassGroupStudentMapper> classGroupStudentMapperPage = new Page<ClassGroupStudentMapper>(classGroupReq.getPageNo(), classGroupReq.getPageSize());
        QueryWrapper<ClassGroupStudentMapper> classGroupStudentMapperQueryWrapper = new QueryWrapper<>();
        classGroupStudentMapperQueryWrapper.lambda().eq(true, ClassGroupStudentMapper::getClassGroupId, classGroupReq.getGroupId());
        IPage<ClassGroupStudentMapper> page = page(classGroupStudentMapperPage, classGroupStudentMapperQueryWrapper);
        Page<StudentListResp> studentListRespPage = new Page<>();
        BeanUtils.copyProperties(page, studentListRespPage);
        List<StudentListResp> list = Lists.newArrayList();
        page.getRecords().forEach(item -> {
            SysUser sysUser = sysUserService.getById(item.getUserId());
            StudentListResp studentRegistrationResp = new StudentListResp()
                    .setStudentName(sysUser.getRealName());
            if (Objects.nonNull(classGroup) && Objects.nonNull(classGroup.getMusicGroupId())) {
                QueryWrapper<MusicGroupStudentFee> musicGroupStudentFeeQueryWrapper = new QueryWrapper<MusicGroupStudentFee>();
                musicGroupStudentFeeQueryWrapper.lambda().eq(true, MusicGroupStudentFee::getMusicGroupId, classGroup.getMusicGroupId())
                        .eq(true, MusicGroupStudentFee::getUserId, item.getUserId());
                MusicGroupStudentFee musicGroupStudentFee = musicGroupStudentFeeService.getOne(musicGroupStudentFeeQueryWrapper);
                Optional.of(musicGroupStudentFee).ifPresent(groupStudentFee -> studentRegistrationResp.
                        setStudentAttendance("连续缺到" + groupStudentFee.getContinuousAbsenteeismTimes() + "次"));
            }
            list.add(studentRegistrationResp);
        });
        studentListRespPage.setRecords(list);
        return PageResponse.success(studentListRespPage);
    }

    @Override
    public List<ClassStudentResp> selectStudentPage(ClassGroupReq classGroupReq) {
        IPage page = new Page(classGroupReq.getPageNo() == null ? 1 : classGroupReq.getPageNo(), classGroupReq.getPageSize() == null ? 10 : classGroupReq.getPageSize());

        List<ClassStudentResp> classStudentResps = this.baseMapper.selectStudentPage(page, classGroupReq);

        return classStudentResps;
    }

    @Override
    public int selectStudentPageCount(ClassGroupReq classGroupReq) {
        return 0;
    }
}
