package com.ym.mec.education.utils;

import java.math.BigDecimal;
import java.sql.Time;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.format.DateTimeFormatter;
import java.util.Calendar;
import java.util.Date;

/**
 * @program: mec
 * @description: 时间工具类
 * @author: xw
 * @create: 2019-09-26 21:22
 */
public class DateUtil {

    private static final String DATE_FORMAT_CHINESE = "MM月dd日";
    private static final String TIME_FORMAT = "HH:mm";
    public static final String DATE_FORMAT = "MM-dd";
    public static final String DATE_FORMAT_HOUR = "MM-dd HH:mm";
    private static ThreadLocal<DateFormat> threadLocal = new ThreadLocal<DateFormat>();

    public static DateFormat getDateFormat(String dateFormat) {
        DateFormat df = new SimpleDateFormat(dateFormat);
        threadLocal.set(df);
        return df;
    }

    public static String date2String(Date date) {
        DateFormat dateFormat = getDateFormat(DATE_FORMAT_CHINESE);
        String format = dateFormat.format(date);
        if (format.startsWith(BigDecimal.ZERO.toString())) {
            format = format.substring(1);
        }
        return format;
    }

    public static String date2String(Date date, String dateFormatPattern) {
        DateFormat dateFormat = getDateFormat(dateFormatPattern);
        return dateFormat.format(date);
    }

    public static String time2String(Time time) {
        String format = time.toLocalTime().format(DateTimeFormatter.ofPattern(TIME_FORMAT));
        return format;
    }

    public static String date2Week(Date date) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        int i = calendar.get(Calendar.DAY_OF_WEEK);
        switch (i) {
            case 1:
                return "星期日";
            case 2:
                return "星期一";
            case 3:
                return "星期二";
            case 4:
                return "星期三";
            case 5:
                return "星期四";
            case 6:
                return "星期五";
            case 7:
                return "星期六";
            default:
                return "";
        }
    }
}
