package com.ym.mec.student.controller;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.page.StudentPayLogQueryInfo;
import com.ym.mec.biz.service.ClassGroupService;
import com.ym.mec.biz.service.MusicGroupService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.security.SecurityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RequestMapping("course")
@Api(tags = "课程服务")
@RestController
public class CourseController extends BaseController {

    @Autowired
    private ClassGroupService classGroupService;
    @Autowired
    private MusicGroupService musicGroupService;
    @Autowired
    private SysUserFeignService sysUserFeignService;

    @ApiOperation("获取学生所购买课程列表")
    @GetMapping(value = "/queryUserMusicGroups")
    public Object queryCoursePage(){
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if(sysUser == null){
            return failed("获取用户信息失败");
        }
        return succeed(classGroupService.queryCoursePage(sysUser.getId()));
    }

    @ApiOperation("获取学员所有聊天群组")
    @GetMapping(value = "/queryUserGroups")
    public Object queryUserGroups(String search){
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if(sysUser == null){
            return failed("获取用户信息失败");
        }
        return succeed(classGroupService.queryUserGroups(sysUser.getId(),search));
    }

    @ApiOperation(value = "根据群编号，获取群组基本信息")
    @GetMapping("/findGroupById")
    public Object findGroupById(String groupId){
        if(StringUtils.isEmpty(groupId)){
            return failed("参数校验错误");
        }
        if(groupId.startsWith("mg")){
            return succeed(musicGroupService.get(groupId.substring(2)));
        }
        return succeed(classGroupService.get(Integer.parseInt(groupId.substring(2))));
    }

    @ApiOperation(value = "分页获取学员购买记录")
    @PostMapping("/queryStudentPayLog")
    public Object queryStudentPayLog(StudentPayLogQueryInfo studentPayLogQueryInfo){
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if(sysUser == null || sysUser.getId() == null){
            return failed("获取用户失败");
        }
        studentPayLogQueryInfo.setUserId(sysUser.getId());
        return succeed(classGroupService.queryStudentPayLog(studentPayLogQueryInfo));
    }
}
