package com.ym.mec.teacher;

import com.spring4all.swagger.EnableSwagger2Doc;
import com.ym.mec.common.redis.service.RedisCache;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.client.loadbalancer.LoadBalanced;
import org.springframework.cloud.openfeign.EnableFeignClients;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.client.RestTemplate;

@SpringBootApplication
@EnableDiscoveryClient
@EnableFeignClients("com.ym.mec")
@MapperScan("com.ym.mec.biz.dal.dao")
@ComponentScan(basePackages = "com.ym.mec")
@Configuration
@EnableSwagger2Doc
public class TeacherApplication {
	public static void main(String[] args) {
		SpringApplication.run(TeacherApplication.class, args);
	}

	@Autowired
	private RedisTemplate<String, Object> redisTemplate;

	@Bean
	@LoadBalanced
	public RestTemplate restTemplate() {
		return new RestTemplate();
	}

	@Bean
	@ConditionalOnBean(RedisTemplate.class)
	public RedisCache<String, Object> redisCache() {
		return new RedisCache<String, Object>(redisTemplate);
	}
}
